@@ -24,7 +24,7 @@
  * Contributor(s):
  *
  * The Original Software is NetBeans. The Initial Developer of the Original
- * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
+ * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
  * Microsystems, Inc. All Rights Reserved.
  *
  * If you wish your version of this file to be governed by only the CDDL
@@ -58,6 +58,7 @@ import org.netbeans.api.autoupdate.OperationContainer.OperationInfo;
 import org.netbeans.api.autoupdate.OperationSupport.Restarter;
 import org.netbeans.api.progress.ProgressHandle;
 import org.netbeans.spi.autoupdate.CustomInstaller;
+import org.netbeans.spi.autoupdate.CustomUninstaller;
 import org.openide.LifecycleManager;
 import org.openide.modules.ModuleInfo;
 
@@ -73,6 +74,7 @@ public abstract class OperationSupportImpl {
     private static final OperationSupportImpl FOR_UNINSTALL = new ForUninstall();
     private static final OperationSupportImpl FOR_DIRECT_UNINSTALL = new ForDirectUninstall();
     private static final OperationSupportImpl FOR_CUSTOM_INSTALL = new ForCustomInstall ();
+    private static final OperationSupportImpl FOR_CUSTOM_UNINSTALL = new ForCustomUninstall ();
     
     private static final Logger LOGGER = Logger.getLogger ("org.netbeans.modules.autoupdate.services.OperationSupportImpl");
     
@@ -100,6 +102,9 @@ public abstract class OperationSupportImpl {
     public static OperationSupportImpl forCustomInstall () {
         return FOR_CUSTOM_INSTALL;
     }
+    public static OperationSupportImpl forCustomUninstall () {
+        return FOR_CUSTOM_UNINSTALL;
+    }
     
     public abstract Boolean doOperation(ProgressHandle progress/*or null*/, OperationContainer<?> container) throws OperationException;
     public abstract void doCancel () throws OperationException;
@@ -111,7 +116,7 @@ public abstract class OperationSupportImpl {
     }
     
     private static class ForEnable extends OperationSupportImpl {
-        public Boolean doOperation(ProgressHandle progress,
+        public synchronized Boolean doOperation(ProgressHandle progress,
                 OperationContainer<?> container) throws OperationException {
             try {
                 if (progress != null) {
@@ -177,7 +182,7 @@ public abstract class OperationSupportImpl {
     private static class ForDisable extends OperationSupportImpl {
         private Collection<File> controlFileForDisable = null;
         private Collection<UpdateElement> affectedModules = null;
-        public Boolean doOperation(ProgressHandle progress,
+        public synchronized Boolean doOperation(ProgressHandle progress,
                 OperationContainer<?> container) throws OperationException {
             try {
                 if (progress != null) {
@@ -253,7 +258,7 @@ public abstract class OperationSupportImpl {
     }
     
     private static class ForDirectDisable extends OperationSupportImpl {
-        public Boolean doOperation(ProgressHandle progress,
+        public synchronized Boolean doOperation(ProgressHandle progress,
                 OperationContainer<?> container) throws OperationException {
             try {
                 if (progress != null) {
@@ -305,7 +310,7 @@ public abstract class OperationSupportImpl {
     private static class ForUninstall extends OperationSupportImpl {
         private Collection<File> files4remove = null;
         private Collection<UpdateElement> affectedModules = null;
-        public Boolean doOperation(ProgressHandle progress,
+        public synchronized Boolean doOperation(ProgressHandle progress,
                 OperationContainer<?> container) throws OperationException {
             try {
                 if (progress != null) {
@@ -333,6 +338,9 @@ public abstract class OperationSupportImpl {
                                 affectedModules.add (moduleImpl.getUpdateElement ());
                             }
                         }
+                        break;
+                    case CUSTOM_HANDLED_COMPONENT :
+
                         break;
                     default:
                         assert false : "Not supported for impl " + updateElementImpl;
@@ -390,7 +398,7 @@ public abstract class OperationSupportImpl {
     }
     
     private static class ForDirectUninstall extends OperationSupportImpl {
-        public Boolean doOperation(ProgressHandle progress,
+        public synchronized Boolean doOperation(ProgressHandle progress,
                 OperationContainer<?> container) throws OperationException {
             try {
                 if (progress != null) {
@@ -466,7 +474,7 @@ public abstract class OperationSupportImpl {
     }
     
     private static class ForInstall extends OperationSupportImpl {
-        public Boolean doOperation(ProgressHandle progress,
+        public synchronized Boolean doOperation(ProgressHandle progress,
                 OperationContainer container) throws OperationException {
             throw new UnsupportedOperationException("Not supported yet.");
         }
@@ -485,7 +493,7 @@ public abstract class OperationSupportImpl {
     }
     
     private static class ForUpdate extends OperationSupportImpl {
-        public Boolean doOperation(ProgressHandle progress,
+        public synchronized Boolean doOperation(ProgressHandle progress,
                 OperationContainer container) throws OperationException {
             throw new UnsupportedOperationException("Not supported yet.");
         }
@@ -504,10 +512,11 @@ public abstract class OperationSupportImpl {
     }
     
     private static class ForCustomInstall extends OperationSupportImpl {
-        public Boolean doOperation(ProgressHandle progress,
+        public synchronized Boolean doOperation(ProgressHandle progress,
                 OperationContainer<?> container) throws OperationException {
+            boolean success = false;
             try {
-                
+
                 List<? extends OperationInfo> infos = container.listAll ();
                 List<NativeComponentUpdateElementImpl> customElements = new ArrayList<NativeComponentUpdateElementImpl> ();
                 for (OperationInfo operationInfo : infos) {
@@ -516,7 +525,6 @@ public abstract class OperationSupportImpl {
                     customElements.add ((NativeComponentUpdateElementImpl) impl);
                 }
                 assert customElements != null : "Some elements with custom installer found.";
-                boolean success = false;
                 for (NativeComponentUpdateElementImpl impl : customElements) {
                     CustomInstaller installer = impl.getInstallInfo ().getCustomInstaller ();
                     assert installer != null : "CustomInstaller must found for " + impl.getUpdateElement ();
@@ -535,10 +543,9 @@ public abstract class OperationSupportImpl {
                     progress.finish ();
                 }
             }
-            
-            // XXX
-            return false;
-            
+
+            return success;
+
         }
         public void doCancel () throws OperationException {
             assert false : "Not supported yet";
@@ -551,6 +558,56 @@ public abstract class OperationSupportImpl {
         public void doRestartLater (Restarter restarter) {
             throw new UnsupportedOperationException ("Not supported yet.");
         }
-        
+
+    }
+
+    private static class ForCustomUninstall extends OperationSupportImpl {
+        public synchronized Boolean doOperation(ProgressHandle progress,
+                OperationContainer<?> container) throws OperationException {
+            boolean success = false;
+            try {
+
+                List<? extends OperationInfo> infos = container.listAll ();
+                List<NativeComponentUpdateElementImpl> customElements = new ArrayList<NativeComponentUpdateElementImpl> ();
+                for (OperationInfo operationInfo : infos) {
+                    UpdateElementImpl impl = Trampoline.API.impl (operationInfo.getUpdateElement ());
+                    assert impl instanceof NativeComponentUpdateElementImpl : "Impl of " + operationInfo.getUpdateElement () + " instanceof NativeComponentUpdateElementImpl.";
+                    customElements.add ((NativeComponentUpdateElementImpl) impl);
+                }
+                assert customElements != null : "Some elements with custom installer found.";
+                for (NativeComponentUpdateElementImpl impl : customElements) {
+                    CustomUninstaller uninstaller = impl.getNativeItem ().getUpdateItemDeploymentImpl ().getCustomUninstaller ();
+                    assert uninstaller != null : "CustomInstaller must found for " + impl.getUpdateElement ();
+                    success = uninstaller.uninstall (impl.getCodeName (),
+                            impl.getSpecificationVersion () == null ? null : impl.getSpecificationVersion ().toString (),
+                            progress);
+                    if (success) {
+                        UpdateUnitImpl unitImpl = Trampoline.API.impl (impl.getUpdateUnit ());
+                        unitImpl.setAsUninstalled ();
+                    } else {
+                        throw new OperationException (OperationException.ERROR_TYPE.UNINSTALL, impl.getDisplayName ());
+                    }
+                }
+            } finally {
+                if (progress != null) {
+                    progress.finish ();
+                }
+            }
+
+            return success;
+
+        }
+        public void doCancel () throws OperationException {
+            assert false : "Not supported yet";
+        }
+
+        public void doRestart (Restarter restarter, ProgressHandle progress) throws OperationException {
+            throw new UnsupportedOperationException ("Not supported yet.");
+        }
+
+        public void doRestartLater (Restarter restarter) {
+            throw new UnsupportedOperationException ("Not supported yet.");
+        }
+
     }
 }
