@@ -102,7 +102,6 @@ public class PluginManagerUI extends javax.swing.JPanel  {
         initTask = Utilities.startAsWorkerThread (new Runnable () {
             public void run () {
                 initialize ();
-                setSelectedTab();
             }
         });
     }
@@ -191,14 +190,15 @@ public class PluginManagerUI extends javax.swing.JPanel  {
         bClose.doClick ();
     }
     
-    public void initialize () {
+    private void initialize () {
         try {
-            units = UpdateManager.getDefault ().getUpdateUnits (Utilities.getUnitTypes ());
+        units = UpdateManager.getDefault ().getUpdateUnits (Utilities.getUnitTypes ());
             // postpone later
             // getLocalDownloadSupport().getUpdateUnits();
             SwingUtilities.invokeAndWait (new Runnable () {
                 public void run () {
                     refreshUnits ();
+                    setSelectedTab ();
                 }
             });
         } catch (InterruptedException ex) {
@@ -209,7 +209,7 @@ public class PluginManagerUI extends javax.swing.JPanel  {
     }
     
     //workaround of #96282 - Memory leak in org.netbeans.core.windows.services.NbPresenter
-    public void unitilialize () {
+    private void unitilialize () {
         Utilities.startAsWorkerThread (new Runnable () {
             public void run () {
                 //ensures that uninitialization runs after initialization
@@ -252,6 +252,9 @@ public class PluginManagerUI extends javax.swing.JPanel  {
     private void setProgressComponentInAwt (JLabel detail, JComponent progressComponent) {
         assert pProgress != null;
         assert SwingUtilities.isEventDispatchThread () : "Must be called in EQ.";
+        
+        progressComponent.setMinimumSize (progressComponent.getPreferredSize ());
+        
         pProgress.setVisible (true);
 
         java.awt.GridBagConstraints gridBagConstraints;
@@ -558,7 +561,6 @@ private void bHelpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:e
             }
             setSelectedTab();
         }        
-        AutoupdateCheckScheduler.runCheckAvailableUpdates (100);
     }
         
     static boolean canContinue (String message) {
