@@ -46,10 +46,11 @@ import java.io.File;
 import java.io.IOException;
 import java.lang.reflect.Method;
 import java.net.URL;
-import java.util.Iterator;
+import java.util.logging.Level;
+import java.util.logging.Logger;
 import javax.swing.SwingUtilities;
 import javax.swing.UIManager;
-import org.netbeans.JarClassLoader;
+import org.netbeans.ProxyURLStreamHandlerFactory;
 import org.netbeans.Stamps;
 import org.netbeans.Util;
 import org.openide.filesystems.FileObject;
@@ -61,7 +62,6 @@ import org.openide.modules.SpecificationVersion;
 import org.openide.util.Exceptions;
 import org.openide.util.Lookup;
 import org.openide.util.NbBundle;
-import org.openide.util.RequestProcessor;
 import org.openide.util.Utilities;
 
 /**
@@ -93,18 +93,9 @@ public final class Main extends Object {
   }
 
 
-  private static boolean nbFactoryInitialized;
   /** Initializes default stream factory */
   public static void initializeURLFactory () {
-    if (!nbFactoryInitialized) {
-        NbURLStreamHandlerFactory fact = new NbURLStreamHandlerFactory();
-        try {
-            java.net.URL.setURLStreamHandlerFactory(fact);
-        } catch (Error e) {
-            fact.registerUsingReflection(e);
-        }
-        nbFactoryInitialized = true;
-    }
+      ProxyURLStreamHandlerFactory.register();
   }
   
   /**
@@ -225,7 +216,7 @@ public final class Main extends Object {
     if (jdkHome == null) {
         jdkHome = System.getProperty("java.home");  // NOI18N
 
-        if (Utilities.isMac()) {
+        if (!Utilities.isMac()) {
             jdkHome += File.separator + "..";  // NOI18N
         }
 
@@ -251,10 +242,6 @@ public final class Main extends Object {
 
 
 // 5. initialize GUI 
-    StartLog.logStart ("XML Factories"); //NOI18N
-    
-    org.netbeans.core.startup.SAXFactoryImpl.install();
-    org.netbeans.core.startup.DOMFactoryImpl.install();
     //Bugfix #35919: Log message to console when initialization of local
     //graphics environment fails eg. due to incorrect value of $DISPLAY
     //on X Windows (Linux, Solaris). In such case IDE will not start
@@ -271,7 +258,6 @@ public final class Main extends Object {
             throw exc;
         }
     }
-    StartLog.logEnd ("XML Factories"); //NOI18N
     
     
 
@@ -294,6 +280,7 @@ public final class Main extends Object {
 	    // -----------------------------------------------------------------------------------------------------
 	    // License check
             if (!handleLicenseCheck()) {
+                deleteRec(new File(CLIOptions.getUserDir())); // #145936
                 TopLogging.exit(0);
             }
 	    // -----------------------------------------------------------------------------------------------------
@@ -349,6 +336,20 @@ public final class Main extends Object {
     // start to store all caches after 15s
     Stamps.getModulesJARs().flush(15000);
   }
+    private static void deleteRec(File f) throws IOException {
+        if (f.isDirectory()) {
+            File[] kids = f.listFiles();
+            if (kids == null) {
+                throw new IOException("Could not list: " + f);
+            }
+            for (File kid : kids) {
+                deleteRec(kid);
+            }
+        }
+        if (!f.delete()) {
+            Logger.getLogger(Main.class.getName()).log(Level.WARNING, "Failed to delete " + f);
+        }
+    }
   
     /** Loads a class from available class loaders. */
     private static final Class getKlass(String cls) {
