@@ -89,7 +89,8 @@ public final class FileObjectFactory {
 
     private FileObjectFactory(final FileInfo fInfo) {
         final File rootFile = fInfo.getFile();
-        assert rootFile.getParentFile() == null;
+        // can be not null for UNC 
+        // assert rootFile.getParentFile() == null;
 
         final BaseFileObj realRoot = create(fInfo);
         root = realRoot;
@@ -195,53 +196,47 @@ public final class FileObjectFactory {
     }
 
 
-    private boolean checkCacheState(boolean exist, File file, Caller caller) {        
-        return checkCacheState(exist, file, caller, false);
-    }
-
-    private boolean checkCacheState(boolean exist, File file, Caller caller, boolean afterRecovering) {
-        if (!exist && (caller.equals(Caller.GetParent) || caller.equals(Caller.ToFileObject))) {
+    /** 
+     * Check cache consistence. It is called only from assertion, so it can be disabled
+     * from command line. Given file should or shouldn't exist according to given 
+     * expectedExists parameter. If the state is different than expected, it prints
+     * warning to console. It can happen when some module doesn't use filesystems API
+     * to handle files but directly uses java.io.File.
+     * @param expectedExists whether given file is expected to exist
+     * @param file checked file
+     * @param caller caller of this method
+     * @return always true.
+     */
+    private boolean checkCacheState(boolean expectedExists, File file, Caller caller) {
+        if (!expectedExists && (caller.equals(Caller.GetParent) || caller.equals(Caller.ToFileObject))) {
             return true;
         }
         if (isWarningEnabled() && caller != null && !caller.equals(Caller.GetChildern)) {
             boolean realExists = file.exists();
-            boolean notsame = exist != realExists;
-            if (!realExists) {
-                //test for broken symlinks
-                File p = file.getParentFile();
-                if (p != null) {
-                    File[] childs = p.listFiles();
-                    if (childs != null && Arrays.asList(childs).contains(p)) {                    
-                        return true;
-                    }
-                }                    
-            }
+            boolean notsame = expectedExists != realExists;
             if (notsame) {
-                if (afterRecovering) {
-                    printWarning(file, Status.RecoverFail);
-                } else {
-                    printWarning(file, Status.NoRecover);
-                }
-            } else {
-                if (afterRecovering) {
-                    printWarning(file, Status.RecoverSuccess);
+                if (!realExists) {
+                    //test for broken symlinks
+                    File p = file.getParentFile();
+                    if (p != null) {
+                        File[] children = p.listFiles();
+                        if (children != null && Arrays.asList(children).contains(file)) {
+                            return true;
+                        }
+                    }                    
                 }
+                printWarning(file);
             }
         }
         return true;
     }
 
-    public static enum Status {
-
-        RecoverSuccess, RecoverFail, NoRecover
-    }
-
     private Integer initRealExists(int initTouch) {
         final Integer retval = new Integer(initTouch);
         return retval;
     }
 
-    private void printWarning(File file, Status stat) {
+    private void printWarning(File file) {
         StringBuilder sb = new StringBuilder("WARNING(please REPORT):  Externally ");
         sb.append(file.exists() ? "created " : "deleted "); //NOI18N
         sb.append(file.isDirectory() ? "folder: " : "file: "); //NOI18N
