@@ -101,11 +101,13 @@ implements Node.Cookie, Serializable, HelpCtx.Provider, Lookup.Provider {
      */
     static final String EA_ASSIGNED_LOADER_MODULE = "NetBeansAttrAssignedLoaderModule"; // NOI18N
 
+    private static final Logger OBJ_LOG = Logger.getLogger(DataObject.class.getName());
+
     /** all modified data objects contains DataObjects.
     * ! Use syncModified for modifications instead !*/
     private static ModifiedRegistry modified = new ModifiedRegistry();
     /** sync modified data (for modification operations) */
-    private static Set<DataObject> syncModified = Collections.synchronizedSet(modified);
+    private static final Set<DataObject> syncModified = Collections.synchronizedSet(modified);
 
     /** Modified flag */
     private boolean modif = false;
@@ -129,7 +131,7 @@ implements Node.Cookie, Serializable, HelpCtx.Provider, Lookup.Provider {
     private static final Object listenersMethodLock = new Object();
     
     /** Lock used for ensuring there will be just one node delegate */
-    private Object nodeCreationLock = new Object();
+    private final Object nodeCreationLock = new Object();
     
     /** Lock for copy/move/rename/etc. operations */
     private static Object synchObject = new Object ();
@@ -162,6 +164,7 @@ implements Node.Cookie, Serializable, HelpCtx.Provider, Lookup.Provider {
     * @param loader loader that created the data object
     */
     private DataObject (FileObject pf, DataObjectPool.Item item, DataLoader loader) {
+        OBJ_LOG.log(Level.FINE, "created {0}", pf); // NOI18N
         this.item = item;
         this.loader = loader;
         item.setDataObject (this);
@@ -180,11 +183,11 @@ implements Node.Cookie, Serializable, HelpCtx.Provider, Lookup.Provider {
     * the original data object, it is protected.
     */
     protected void dispose () {
-        DataObjectPool.Item item = this.item;
+        DataObjectPool.Item i = item;
         
-        if (item != null) {
-            item.deregister (true);
-            item.setDataObject(null);
+        if (i != null) {
+            i.deregister (true);
+            i.setDataObject(null);
             firePropertyChange (PROP_VALID, Boolean.TRUE, Boolean.FALSE);
         }
     }
@@ -518,6 +521,7 @@ implements Node.Cookie, Serializable, HelpCtx.Provider, Lookup.Provider {
         return getPrimaryFile ().getName ();
     }
 
+    @Override
     public String toString () {
         return super.toString () + '[' + getPrimaryFile () + ']';
     }
@@ -1075,6 +1079,32 @@ implements Node.Cookie, Serializable, HelpCtx.Provider, Lookup.Provider {
         return NbBundle.getMessage (DataObject.class, name);
     }
     
+    /** Factory interface for converting file object to data objects. Read
+     * more about the layer based registrations in 
+     * <a href="@TOP@/org/openide/loaders/doc-files/api.html#register"/>separate document</a>.
+     * @since 7.0
+     */
+    public static interface Factory {
+        /** Find a data object appropriate to the given file object--the meat of this class.
+        * The loader can add all files it has recognized into the <CODE>recognized</CODE>
+        * buffer. Then all these files will be excluded from further processing.
+        *
+        * @param fo file object to recognize
+        * @param recognized recognized file buffer
+        * @exception DataObjectExistsException if the data object for the
+        *    primary file already exists
+        * @exception IOException if the object is recognized but cannot be created
+        * @exception InvalidClassException if the class is not instance of
+        *    {@link #getRepresentationClass}
+        *
+        * @return suitable data object or <CODE>null</CODE> if the handler cannot
+        *   recognize this object (or its group)
+        * @see DataLoader
+        */
+        public DataObject findDataObject(FileObject fo, Set<? super FileObject> recognized)
+        throws IOException;
+    }
+    
     /** Interface for objects that can contain other data objects.
      * For example DataFolder and DataShadow implement this interface
      * to allow others to access the contained objects in uniform maner
