@@ -45,11 +45,22 @@ import org.openide.filesystems.*;
 import java.io.*;
 import java.lang.ref.Reference;
 import java.lang.ref.WeakReference;
+import java.security.Permission;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.Map;
+import java.util.Set;
+import java.util.TreeSet;
 import java.util.logging.Level;
+import junit.framework.Assert;
 import org.netbeans.junit.Log;
 import org.openide.cookies.*;
 import org.openide.nodes.Node;
 import org.openide.util.RequestProcessor;
+import org.openide.xml.XMLUtil;
+import org.xml.sax.InputSource;
+import org.xml.sax.SAXParseException;
 
 /**
  *
@@ -64,14 +75,20 @@ public class XMLDataObjectTest extends org.netbeans.junit.NbTestCase {
         super (name);
     }
 
+//    public static Test suite() {
+//        return new XMLDataObjectTest("testWrongUTFCharacer");
+//    }
+
+    @Override
     protected void setUp () throws Exception {
+        clearWorkDir();
+        
         log = Log.enable("org.openide.loaders", Level.WARNING);
         
         super.setUp ();
         System.setProperty ("org.openide.util.Lookup", "org.openide.loaders.XMLDataObjectTest$Lkp");
         String fsstruct [] = new String [] {
         };
-        TestUtilHid.destroyLocalFileSystem (getName());
         FileSystem fs = TestUtilHid.createLocalFileSystem (getWorkDir(), fsstruct);
         data = FileUtil.createData (
             fs.getRoot (),
@@ -87,8 +104,26 @@ public class XMLDataObjectTest extends org.netbeans.junit.NbTestCase {
         
         p.close ();
         lock.releaseLock ();
+
+        // initialize the document before we start to measure the access
+        assertNotNull("Constructor ready", XMLDataObject.cnstr);
+        CountingSecurityManager.initialize();
+
+        assertParse(false, false);
+        assertParse(false, true);
+        assertParse(true, false);
+        assertParse(true, true);
+    }
+
+    private static void assertParse(boolean validate, boolean namespace) throws Exception {
+        try {
+            XMLUtil.parse(new InputSource(new ByteArrayInputStream(new byte[0])), validate, namespace, null, null);
+        } catch (SAXParseException ex) {
+        }
+        CountingSecurityManager.assertMembers(1);
     }
     
+    @Override
     protected void tearDown () throws Exception {
         super.tearDown ();
         TestUtilHid.destroyLocalFileSystem (getName());
@@ -117,7 +152,7 @@ public class XMLDataObjectTest extends org.netbeans.junit.NbTestCase {
         org.w3c.dom.Element e = doc.getDocumentElement ();
         assertNotNull ("Document parsed", doc);
         
-        assertEquals ("status is ok", xml.STATUS_OK, xml.getStatus ());
+        assertEquals ("status is ok", XMLDataObject.STATUS_OK, xml.getStatus ());
         
         assertNotNull("Has open cookie", xml.getCookie(OpenCookie.class));
         assertNotNull("Has open cookie in lookup", xml.getLookup().lookup(OpenCookie.class));
@@ -129,6 +164,9 @@ public class XMLDataObjectTest extends org.netbeans.junit.NbTestCase {
         doc = null;
         e = null;
         assertGC("Data object has to be garbage collectable", ref);
+
+
+        CountingSecurityManager.assertMembers(0);
     }
 
     public void testCookieIsUpdatedWhenContentChanges () throws Exception {
@@ -166,6 +204,8 @@ public class XMLDataObjectTest extends org.netbeans.junit.NbTestCase {
         }
         assertEquals ("One change fired when the file was written", 1, pcl.cnt);
         assertNotNull ("There is an cookie", obj.getCookie (org.openide.cookies.InstanceCookie.class));
+
+        CountingSecurityManager.assertMembers(0);
     }
     
     public void testToolbarsAreBrokenAsTheLookupIsClearedTooOftenIssue41360 () throws Exception {
@@ -226,6 +266,31 @@ public class XMLDataObjectTest extends org.netbeans.junit.NbTestCase {
             XMLDataObject.registerInfo (id, null);
             lck.releaseLock ();
         }
+        CountingSecurityManager.assertMembers(1);
+    }
+
+    public void testWrongUTFCharacer() throws Exception {
+        FileLock lck;
+        DataObject obj;
+
+        FileObject d = data.getParent().createData("wrongutfchar.xml");
+
+        lck = d.lock();
+
+
+        OutputStream os = d.getOutputStream(lck);
+        os.write(0xc5); // multibyte
+        os.write(0x00); // wrong char after multibyte
+        os.close();
+
+        obj = DataObject.find(d);
+
+        XMLDataObject xml = (XMLDataObject)obj;
+        String id = xml.getDocument().getDoctype().getPublicId();
+        assertEquals("No ID", null, id);
+        
+        assertEquals("No warnings\n" + log, 0, log.length());
+        CountingSecurityManager.assertMembers(0);
     }
     
     
@@ -395,5 +460,140 @@ public class XMLDataObjectTest extends org.netbeans.junit.NbTestCase {
             }
         }
     } // end of PCL
+
+
+    static final class CountingSecurityManager extends SecurityManager {
+        public static void initialize() {
+            if (System.getSecurityManager() instanceof CountingSecurityManager) {
+                // ok
+            } else {
+                System.setSecurityManager(new CountingSecurityManager());
+            }
+            members.clear();
+        }
+
+        static void assertMembers(int cnt) {
+            int myCnt = 0;
+            StringWriter w = new StringWriter();
+            PrintWriter p = new PrintWriter(w);
+            Set<Who> m;
+            synchronized (members) {
+                m = new TreeSet<Who>(members.values());
+            }
+            for (Who wh : m) {
+                if (wh.isIgnore()) {
+                    continue;
+                }
+
+                myCnt += wh.count;
+                wh.printStackTrace(p);
+                wh.count = 0;
+            }
+            if (myCnt > cnt) {
+                Assert.fail("Expected at much " + cnt + " reflection efforts, but was: " + myCnt + "\n" + w);
+            }
+        }
+
+        static Map<Class,Who> members = Collections.synchronizedMap(new HashMap<Class, Who>());
+        @Override
+        public void checkMemberAccess(Class<?> clazz, int which) {
+            if (clazz == null) {
+                assertMembers(which);
+            }
+
+            Who w = members.get(clazz);
+            if (w == null) {
+                w = new Who(clazz);
+                members.put(clazz, w);
+            }
+            w.count++;
+        }
+
+        private static class Who extends Exception implements Comparable<Who> {
+            int hashCode;
+            final Class<?> clazz;
+            int count;
+
+            public Who(Class<?> who) {
+                super("");
+                this.clazz = who;
+            }
+
+            @Override
+            public void printStackTrace(PrintWriter s) {
+                s.println("Members of class " + clazz.getName() + " initialized " + count + " times");
+                super.printStackTrace(s);
+            }
+
+            @Override
+            public int hashCode() {
+                if (hashCode != 0) {
+                    return hashCode;
+                }
+                hashCode = clazz.hashCode();
+                for (StackTraceElement stackTraceElement : getStackTrace()) {
+                    hashCode = hashCode * 2 + stackTraceElement.hashCode();
+                }
+                return hashCode;
+            }
+
+            @Override
+            public boolean equals(Object obj) {
+                if (obj == null) {
+                    return false;
+                }
+                if (getClass() != obj.getClass()) {
+                    return false;
+                }
+                final Who other = (Who) obj;
+                if (this.clazz != other.clazz) {
+                    return false;
+                }
+                if (this.hashCode() != other.hashCode()) {
+                    return false;
+                }
+                return Arrays.equals(getStackTrace(), other.getStackTrace());
+            }
+
+            public int compareTo(Who o) {
+                if (o == this) {
+                    return 0;
+                }
+                if (o.count < this.count) {
+                    return -1;
+                }
+                if (o.count > this.count) {
+                    return 1;
+                }
+                return this.clazz.getName().compareTo(o.clazz.getName());
+            }
+
+            private boolean isIgnore() {
+                for (StackTraceElement stackTraceElement : getStackTrace()) {
+                    if (stackTraceElement.getClassName().startsWith("org.openide.loaders.XMLDataObject$")) {
+                        return false;
+                    }
+                    if (stackTraceElement.getClassName().equals("org.openide.loaders.XMLDataObject")) {
+                        return false;
+                    }
+                    if (stackTraceElement.getClassName().equals("org.openide.nodes.FilterNode")) {
+                        return true;
+                    }
+                    if (stackTraceElement.getClassName().equals("org.openide.loaders.DataNode")) {
+                        return true;
+                    }
+                }
+                return true;
+            }
+        }
+
+        @Override
+        public void checkPermission(Permission perm, Object context) {
+        }
+
+        @Override
+        public void checkPermission(Permission perm) {
+        }
+    }
     
 }
