@@ -44,11 +44,17 @@ package org.openide.xml;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.CharConversionException;
+import java.io.File;
 import java.io.IOException;
 import java.io.StringReader;
 import java.lang.ref.WeakReference;
+import javax.xml.XMLConstants;
 import javax.xml.parsers.DocumentBuilderFactory;
+import javax.xml.transform.stream.StreamSource;
+import javax.xml.validation.Schema;
+import javax.xml.validation.SchemaFactory;
 import org.netbeans.junit.NbTestCase;
+import org.openide.util.test.TestFileUtils;
 import org.w3c.dom.CDATASection;
 import org.w3c.dom.Document;
 import org.w3c.dom.DocumentType;
@@ -154,6 +160,56 @@ public class XMLUtilTest extends NbTestCase {
             return new InputSource(new StringReader(data));
         }
     }
+
+    public void testValidate() throws Exception {
+        Element r = XMLUtil.createDocument("root", "some://where", null, null).getDocumentElement();
+        r.setAttribute("hello", "there");
+        SchemaFactory f = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
+        String xsd =
+                "<xsd:schema xmlns:xsd='http://www.w3.org/2001/XMLSchema' targetNamespace='some://where' xmlns='some://where' elementFormDefault='qualified'>\n" +
+                " <xsd:element name='root'>\n" +
+                "  <xsd:complexType>\n" +
+                "   <xsd:attribute name='hello' type='xsd:NMTOKEN' use='required'/>\n" +
+                "  </xsd:complexType>\n" +
+                " </xsd:element>\n" +
+                "</xsd:schema>\n";
+        Schema s = f.newSchema(new StreamSource(new StringReader(xsd)));
+        XMLUtil.validate(r, s);
+        r.setAttribute("goodbye", "now");
+        try {
+            XMLUtil.validate(r, s);
+            fail();
+        } catch (SAXException x) {/*OK*/}
+        // Make sure Java #6529766 is fixed (no longer any need for fixupNoNamespaceAttrs):
+        String xml = "<root xmlns='some://where'/>";
+        r = XMLUtil.parse(new InputSource(new StringReader(xml)), false, true, null, null).getDocumentElement();
+        r.setAttribute("hello", "there");
+        XMLUtil.validate(r, s);
+        r.setAttribute("goodbye", "now");
+        try {
+            XMLUtil.validate(r, s);
+            fail();
+        } catch (SAXException x) {/*OK*/}
+        // #146081: xml:base attributes get inserted sometimes and can mess up validation.
+        xsd =
+                "<xsd:schema xmlns:xsd='http://www.w3.org/2001/XMLSchema' targetNamespace='some://where' xmlns='some://where' elementFormDefault='qualified'>\n" +
+                " <xsd:element name='root'>\n" +
+                "  <xsd:complexType>\n" +
+                "   <xsd:sequence>\n" +
+                "    <xsd:element name='hello' type='xsd:NMTOKEN'/>\n" +
+                "   </xsd:sequence>\n" +
+                "  </xsd:complexType>\n" +
+                " </xsd:element>\n" +
+                "</xsd:schema>\n";
+        s = f.newSchema(new StreamSource(new StringReader(xsd)));
+        File d = getWorkDir();
+        File main = new File(d, "main.xml");
+        File ent = new File(d, "ent.xml");
+        TestFileUtils.writeFile(main, "<!DOCTYPE root [<!ENTITY ent SYSTEM 'ent.xml'>]> <root xmlns='some://where'>&ent;</root>");
+        TestFileUtils.writeFile(ent, "<hello xmlns='some://where'>there</hello>");
+        r = XMLUtil.parse(new InputSource(main.toURI().toString()), false, true, null, null).getDocumentElement();
+        XMLUtil.validate(r, s);
+    }
     
     public void testToAttributeValue() throws IOException {
         String result = null;
